@@ -13,6 +13,7 @@
 #include <QListWidget>
 
 #include <KPluginMetaData>
+#include <kpluginmetadata.h>
 #include <kservice.h>
 
 class QPushButton;
@@ -23,7 +24,7 @@ class KService;
 class KServiceListItem : public QListWidgetItem
 {
 public:
-    KServiceListItem(const KService::Ptr &pService, int kind);
+    explicit KServiceListItem(const KService::Ptr &pService);
     QString storageId;
     QString desktopPath;
     QString localPath;
@@ -32,11 +33,12 @@ public:
 class PluginListItem : public QListWidgetItem
 {
 public:
-    PluginListItem(const KPluginMetaData &metaData);
+    PluginListItem(const KPluginMetaData &data);
+    KPluginMetaData metaData;
 };
 
 /**
- * This widget holds a list of services, with 5 buttons to manage it.
+ * This widget holds a list of services, with 6 buttons to manage it.
  * It's a separate class so that it can be used by both tabs of the
  * module, once for applications and once for services.
  * The "kind" is determined by the argument given to the constructor.
@@ -45,16 +47,19 @@ class KServiceListWidget : public QGroupBox
 {
     Q_OBJECT
 public:
-    enum {
+    enum Kind {
         SERVICELIST_APPLICATIONS,
         SERVICELIST_SERVICES,
     };
-    explicit KServiceListWidget(int kind, QWidget *parent = nullptr);
+    explicit KServiceListWidget(Kind kind, QWidget *parent = nullptr);
 
     void setMimeTypeData(MimeTypeData *item);
 
+    void allowMultiApply(bool allow);
+
 Q_SIGNALS:
     void changed(bool);
+    void multiApply(int kind);
 
 protected Q_SLOTS:
     void promoteService();
@@ -62,17 +67,20 @@ protected Q_SLOTS:
     void addService();
     void editService();
     void removeService();
+    void applyTo();
     void enableMoveButtons();
 
 protected:
     void updatePreferredServices();
 
 private:
-    int m_kind;
+    Kind m_kind;
     QListWidget *servicesLB;
     QPushButton *servUpButton, *servDownButton;
     QPushButton *servNewButton, *servEditButton, *servRemoveButton;
+    QPushButton *servApplyToButton;
     MimeTypeData *m_mimeTypeData;
+    bool m_allowMultiApply;
 };
 
 #endif
